# Micro servicios v2.0.0

El micro servicio hereda de *Worker* o *Fork* dependiendo del elemento que se quiera crear.

> ## NOTAS.
> Las posibles acciones que tienen un micro servicio son:
> * List
> * Create
> * Get
> * Update
> * Delete
> 
> Esta son definidas utilizando decoradores tal como se muestra en el ejemplo.

### Ejemplo:

``` typescript
import {
    ErrorMs,
    Micorservices as Ms,
    Types,
    Worker,
} from '@coppel/microservices';

@Ms.define({
    App: 'NombreApp',
    Name: 'NombreWorker',
    Type: Types.Worker,
    Version: '1.0.0',
})
class MiWorker extends Worker {
    
    /**
     *  Errores que puede tener el servicio.
     */
    private Error: ErrorMs[] = [
        {"-97" : "Hay un problema en el servicio."},
        {"-98" : "Otro Error Registrado"}
    ];

    @Ms.List
    public metodoList(data: objet, auth: object): void {
        // Listar
    }

    @Ms.Create
    public crear(data: objet, auth: object): void {
        // Registrar
    }

    @Ms.Get
    public recuperar(data: objet, auth: object): void {
        // Recuperar un elemento
    }

    @Ms.Update
    public actualizar(data: objet, auth: object): void {
        // Actualizar un elemento
    }

    @Ms.Delete
    public eliminar(data: objet, auth: object): void {
        // Eliminar un elemento
    }

}

```

> ### NOTAS:
> Las clases son auto ejecutables, no se requiere que se implemente una inicializacio/start o similar para poner en marcha el servicio.
>
> Para la indicacion de la cola de mensajes se utiliza la variable de ambiente *KAFKA*.


## Diseño Legado.

Si lo que se quiere es crear un micro servicio basado en el diseño de la version v1.* se utiliza la definicion de *Listener*, como se indica en el siguiente ejemplo:

``` typescript
import {
    ErrorMs,
    Micorservices as Ms,
    Types,
    Worker,
} from '@coppel/microservices';

@Ms.define({
    App: 'NombreApp',
    Name: 'NombreWorker',
    Type: Type.Worker,
    Version: '1.0.0',
})
class MiWorker extends Worker {
    
    /**
     *  Errores que puede tener el servicio.
     */
    private Errors: ErrorMs[] = [
        {"-97" : "Hay un problema en el servicio."},
        {"-98" : "Otro Error Registrado"}
    ];

    @Ms.Listener
    public metodoList(data: objet, auth: object): void {
        // Escuchar todos los eventos
    }

}

```

## Retorno de valores

Para retornar una respuesta simplemente se debe regresar el valor de la funcion:
```typescript
  @MS.Get
  public miFuncion(data: object, auth: object): object {
       // Retorno de un objeto
       return {
           Clave: 'Valor',
       };
   }
 ```

> ### NOTAS:
>
> En caso de que se requiera regresar un valor con una promesa:
> ```typescript
>   @MS.Get
>   public miFuncion(data: object, auth: object): Promise<object> {
>       // Retornar la promesa
>       return new Promise( (resolve: any, fail: any) => {
>           resolve({ Clave: 'Valor' });
>       });
>   }
> ```
>
> El core detecta la promesa, y espera que se resuelva para retornar la respuesta a la peticion inicial.

## Errores

Para la notificacion de errores, se implementa la utilizada por el lenguaje, llamando a la propiedad heredada Errors, como muestra el ejemplo.

```typescript
  @MS.Get
  public miFuncion(data: object, auth: object): object {
       // Ejecutar el error.
       throw this.Error('-34'); // definido antes.
   }
 ```

 Esto ejecutara la excepcion que es escuchada, por el core, notificando a la peticion original.

 > ### NOTAS:
 >
 > Cuando se utiliza una promesa se puede implementar el llamdo a la excepcion como el ejemplo anterior o de la siguiente manera.
 >
> ```typescript
>   @MS.Get
>   public miFuncion(data: object, auth: object): Promise<object> {
>       // Retornar la promesa
>       return new Promise( (resolve: any, fail: any) => {
>           fail(this.Error('-34'));
>       });
>   }
> ```
>
> Ambas opciones generar un error que es enviado por el core.