"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("./base");
const util_1 = require("./util");
class Worker extends base_1.default {
    /**
     * Metodo para procesar los mensajes entrantes
     * @param data Datos de la peticion
     */
    message(data, Method) {
        return new Promise((rr, ff) => {
            // Ejecutar la funcion
            try {
                const resp = Method.apply({}, [
                    data.data,
                    data.headers.Authorizacion,
                    data.metadata.uuid,
                ]);
                // Validar si la respuesta es una promesa y esperar a resolverla
                if (util_1.default.isPromise(resp)) {
                    // Esperar que finalice
                    resp.then((response) => {
                        // Regresar el contenido
                        rr(this.formatResponse(data, response, 0));
                    }).catch((error) => {
                        // Indicar que fue error
                        ff(this.formatResponse(data, null, error.code));
                    });
                }
                else {
                    // Regresar la respuesta
                    rr(this.formatResponse(data, resp, 0));
                }
            }
            catch (error) {
                // Regresar el error
                ff(this.formatResponse(data, null, error.code));
            }
        });
    }
    /**
     * Metodo para dar formato a los mensajes de salida.
     * @param data Datos a enviar
     * @param arg Argumentos extra
     */
    formatResponse(Data, ...arg) {
        // Variable para conocer si es un error o no
        let isError = false;
        // Validar si es un error
        if (arg[1] < 0) {
            isError = true;
        }
        // Objecto base de respuesta
        const response = {
            data: Data.data,
            headers: Data.headers,
            metadata: Data.metadata,
            response: {
                data: {
                    response: arg[0],
                },
                meta: {
                    id_transaction: Data.metadata.id_transaction,
                    status: isError ? 'ERROR' : 'SUCCESS',
                },
            },
        };
        // Regresar todo
        return {
            errorCode: arg[1],
            response,
        };
    }
}
exports.default = Worker;
