"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
class Util {
    /**
     * Metodo para validar si un elemento es una promesa
     * @param obj Objecto a validar
     */
    static isPromise(obj) {
        return obj && (typeof obj === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
    }
    /**
     * Get a nested object property value
     * Checks for a nested object property and returns its value.
     * @param   {object}    obj     An object given
     * @param   {string}    key     The needed key
     * @return  {string}            The value of that key.
     */
    get(key, target = null) {
        if (key) {
            return key.split('.').reduce((o, x) => {
                return (typeof o === 'undefined' || o === null) ? o : o[x];
            }, target ? target : this);
        }
        return undefined;
    }
    /**
     * Get a nested object property value
     * Checks for a nested object property and returns its value.
     * @param   {object}    obj     An object given
     * @param   {string}    key     The needed key
     * @return  {string}            The value of that key.
     */
    has(key) {
        return util_1.isUndefined(this.get(key));
    }
    /**
     * Metodo para la conversion de un objecto en array-like
     * @param   {object}    data    Objecto a convertir
     * @return  {object}            Resultado de la conversion
     */
    arrayLike(data) {
        const result = {};
        let cont = 0;
        for (const v of data) {
            result[cont] = v;
            cont++;
        }
        return result;
    }
}
exports.default = Util;
